#ifndef OPT_VERTEX_BUFFER_H
#define OPT_VERTEX_BUFFER_H

#include "../OpenGL/glExtensions.h"

/**
 * Fast, Video (or AGP) memory storage of vertex information
 * Sub-classes optimized with vendor extensions (can you say D3D?)
 * Default implementation works with vertex data as-is in regular
 * memory
 */
class OptVertexBuffer
{
public :
	virtual ~OptVertexBuffer()=0;
	virtual void setVertexPointer( GLint count, GLenum type, GLsizei stride, GLsizei offset)=0;
	virtual void setNormalPointer( GLenum type, GLsizei stride, GLsizei offset)=0;
	virtual void setTexCoordPointer( GLint count, GLenum type, GLsizei stride, GLsizei offset)=0;
    virtual void map();
	virtual void unmap();
};

class OptVertexBufferFactory
{
public :
	virtual OptVertexBuffer* getBuffer( GLvoid* vertices, GLsizei size)=0;
	virtual void release()=0;
	static OptVertexBufferFactory& getFactory( GLsizei typical_size, GLsizei max_total_size);
};
#endif